@@ -409,9 +409,10 @@ impl PruneIndex {
 }
 
 /// Task to be executed by a `PrunePlan` on Packs
-#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Serialize)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Serialize, Default)]
 pub enum PackToDo {
     // TODO: Add documentation
+    #[default]
     Undecided,
     /// The pack should be kept
     Keep,
@@ -429,12 +430,6 @@ pub enum PackToDo {
     Delete,
 }
 
-impl Default for PackToDo {
-    fn default() -> Self {
-        Self::Undecided
-    }
-}
-
 /// A pack which is to be pruned
 #[derive(Debug)]
 struct PrunePack {
@@ -1532,7 +1527,7 @@ impl PackInfo {
 
         if let Some(first_needed) = first_needed {
             // The pack is actually needed.
-            // We reprocess the blobs up to the first needed one and mark all blobs which are genarally needed as used.
+            // We reprocess the blobs up to the first needed one and mark all blobs which are generally needed as used.
             for blob in &pack.blobs[..first_needed] {
                 match used_ids.get_mut(&blob.id) {
                     None | Some(0) => {} // already correctly marked
