@@ -23,7 +23,7 @@ use crate::{
     backend::{FileType, FindInBackend, decrypt::DecryptReadBackend},
     blob::tree::TreeId,
     error::{ErrorKind, RusticError, RusticResult},
-    id::{FindUniqeMultiple, FindUniqueResults, constants::HEX_LEN},
+    id::{FindUniqueMultiple, FindUniqueResults, constants::HEX_LEN},
     impl_repofile,
     progress::Progress,
     repofile::RepoFile,
@@ -473,7 +473,7 @@ impl SnapshotRequests {
                 SnapshotRequest::Latest(n) => {
                     max_n_latest = Some(max_n_latest.unwrap_or_default().max(*n));
                 }
-                SnapshotRequest::StartsWith(s) => starts_with.push(s.to_string()),
+                SnapshotRequest::StartsWith(s) => starts_with.push(s.clone()),
                 SnapshotRequest::Id(id) => ids.push(*id),
             }
         }
