@@ -33,7 +33,7 @@ use crate::{
     commands::{
         self,
         backup::BackupOptions,
-        check::{CheckOptions, check_repository},
+        check::{CheckOptions, CheckResults, check_repository},
         config::ConfigOptions,
         copy::CopySnapshot,
         forget::{ForgetGroups, KeepOptions},
@@ -938,7 +938,7 @@ impl<P: ProgressBars, S: Open> Repository<P, S> {
         &self,
         ids: &[String],
         group_by: SnapshotGroupCriterion,
-        filter: impl FnMut(&SnapshotFile) -> bool,
+        filter: impl FnMut(&SnapshotFile) -> bool + Send + Sync,
     ) -> RusticResult<Vec<(SnapshotGroup, Vec<SnapshotFile>)>> {
         commands::snapshots::get_snapshot_group(self, ids, group_by, filter)
     }
@@ -959,7 +959,7 @@ impl<P: ProgressBars, S: Open> Repository<P, S> {
     /// # Returns
     ///
     /// If `id` is (part of) an `Id`, return this snapshot.
-    /// If `id` is "latest", return the latest snapshot respecting the giving filter.
+    /// If `id` is "latest" or "latest~N", return the latest (or Nth latest) snapshot respecting the giving filter.
     pub fn get_snapshot_from_str(
         &self,
         id: &str,
@@ -971,6 +971,35 @@ impl<P: ProgressBars, S: Open> Repository<P, S> {
         Ok(snap)
     }
 
+    /// Get a single snapshot
+    ///
+    /// # Arguments
+    ///
+    /// * `ids` - The ids of the snapshot to get
+    ///   * each `id` can use an actual (short) id "01a2b3c4" or "latest" or "latest~N"
+    /// * `filter` - The filter to use
+    ///
+    /// # Errors
+    ///
+    /// * If the string is not a valid hexadecimal string
+    /// * If no id could be found.
+    /// * If the id is not unique.
+    ///
+    /// # Returns
+    ///
+    /// If `id` is (part of) an `Id`, return this snapshot.
+    /// If `id` is "latest" or "latest~N", return the latest (or Nth latest) snapshot respecting the giving filter.
+    pub fn get_snapshots_from_strs<T: AsRef<str>>(
+        &self,
+        ids: &[T],
+        filter: impl FnMut(&SnapshotFile) -> bool + Send + Sync,
+    ) -> RusticResult<Vec<SnapshotFile>> {
+        let p = self.pb.progress_counter("getting snapshots...");
+        let snaps = SnapshotFile::from_strs(self.dbe(), ids, filter, &p)?;
+        p.finish();
+        Ok(snaps)
+    }
+
     /// Get the given snapshots.
     ///
     /// # Arguments
@@ -1104,7 +1133,7 @@ impl<P: ProgressBars, S: Open> Repository<P, S> {
         &self,
         keep: &KeepOptions,
         group_by: SnapshotGroupCriterion,
-        filter: impl FnMut(&SnapshotFile) -> bool,
+        filter: impl FnMut(&SnapshotFile) -> bool + Send + Sync,
     ) -> RusticResult<ForgetGroups> {
         commands::forget::get_forget_snapshots(self, keep, group_by, filter)
     }
@@ -1189,16 +1218,14 @@ impl<P: ProgressBars, S: Open> Repository<P, S> {
     /// # Panics
     ///
     // TODO: Document panics
-    pub fn check(&self, opts: CheckOptions) -> RusticResult<()> {
+    pub fn check(&self, opts: CheckOptions) -> RusticResult<CheckResults> {
         let trees = self
             .get_all_snapshots()?
             .into_iter()
             .map(|snap| snap.tree)
             .collect();
 
-        check_repository(self, opts, trees)?;
-
-        Ok(())
+        check_repository(self, opts, trees)
     }
 
     /// Check the repository and given trees for errors or inconsistencies
@@ -1213,7 +1240,11 @@ impl<P: ProgressBars, S: Open> Repository<P, S> {
     /// # Panics
     ///
     // TODO: Document panics
-    pub fn check_with_trees(&self, opts: CheckOptions, trees: Vec<TreeId>) -> RusticResult<()> {
+    pub fn check_with_trees(
+        &self,
+        opts: CheckOptions,
+        trees: Vec<TreeId>,
+    ) -> RusticResult<CheckResults> {
         check_repository(self, opts, trees)
     }
 
